/*
Dozen Days of Words episode 6: Word Slide
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package
{
	import flash.display.MovieClip;
	import flash.events.*;
	import flash.text.TextField;
	import flash.ui.Keyboard;

	import com.blazinggames.dozendays.*;
	import com.blazinggames.dozendays.words.*;
	import com.blazinggames.dozendays.words.slide.*;

	[SWF(width="640", height="480", backgroundColor="#000000")]
	
	public class WordSlide extends MovieClip implements IGameMode
	{
// ***** CONSTANTS ***********************************************************
		public static const PAGE1_TEXT:String = 
"This game was created for Blazing Games Inc. by Billy D. Spelchan "+
"(the person writing this).\n\n"+
"This game is part of the Dozen Days of Words series which is a collection of "+
"Games that are created in under 24 hours of working time that revolve around "+
"the creation of computer versions of word games. This episode is a The " +
"fifth episode is a semi-original game. is a semi-original game that is " +
"essentially a combination of my Sliding Puzzle game and the Growing Words "+
"game that made up episode 4. Words are formed by sliding letters into a " +
"single empty tile. The words will continually be added to the game board. " +
"The thing that prevents the game from going on forever is a time limit that " +
"is increased every time a word is removed.";

		public static const PAGE2_TEXT:String = 
"This program is free software; you can redistribute it and/or modify "+
"it under the terms of the GNU General Public License as published by "+
"the Free Software Foundation; either version 2 of the License, or "+
"(at your option) any later version.\n\n"+
"This program is distributed in the hope that it will be useful, "+
"but WITHOUT ANY WARRANTY; without even the implied warranty of "+
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "+
"GNU General Public License for more details.\n\n"+
"You should have received a copy of the GNU General Public License "+
"along with this program; if not, write to the Free Software "+
"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA";

		public static const MODE_NONE:int = 0;
		public static const MODE_TITLE:int = 1;
		public static const MODE_GAME:int = 2;
		public static const MODE_ABOUT:int = 3;

// ***** VARIABLES ***********************************************************
		
		private var _mode:int;
		private var _title:SlideTitle;
		private var _game:SlideGame;
		private var _about:About;
		
// ***** CONSTRUCTOR *********************************************************

		public function WordSlide()
		{
			_mode = MODE_NONE;
			_title = new SlideTitle(this);
			_game = new SlideGame(this);

			_about = new About(this);
			_about.setCaption("About Word Slide");
			_about.addPage(PAGE1_TEXT);
			_about.addPage(PAGE2_TEXT);
			changeMode(MODE_TITLE);
			stage.addEventListener(Event.ENTER_FRAME, handleEnterFrame);
			stage.addEventListener(KeyboardEvent.KEY_UP, handleKeyUp);
			stop();
		}
		
// ***** Functions - mode handlers ******************************************		
		
		public function changeMode(n:int):void
		{
			// remove old mode
			switch(_mode) {
				case MODE_NONE:
					break;
				case MODE_GAME:
					removeChild(_game);
					break;
				case MODE_TITLE:
					removeChild(_title);
					break;
				case MODE_ABOUT:
					removeChild(_about);
					break;
			}
			
			// change to the new mode
			switch(n) {
				case MODE_NONE:
					break;
				case MODE_GAME:
					_game.startGame(_title.getPlayMode());
					addChildAt(_game, 0);
					break;
				case MODE_TITLE:
					addChildAt(_title, 0);
					break;
				case MODE_ABOUT:
					addChildAt(_about, 0);
					break;
			}
			
			_mode = n;
		}

		public function handleEnterFrame(evt:Event):void
		{
			if (_mode == MODE_GAME)
				_game.updateTime();
		}

		public function handleKeyUp(evt:KeyboardEvent):void
		{
			if (_mode != MODE_GAME)
				return;
			switch (evt.keyCode) {
				case Keyboard.UP:
					_game.slideUp();
					break;
				case Keyboard.LEFT:
					_game.slideLeft();
					break;
				case Keyboard.RIGHT:
					_game.slideRight();
					break;
				case Keyboard.DOWN:
					_game.slideDown();
					break;
				
			}
		}

	}
}
