/*
Dozen Days of Words episode 6: Word Slide
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.words.slide
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.media.Sound;
	import flash.media.SoundMixer;
	import flash.media.SoundTransform;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	import flash.utils.getTimer;

	import com.blazinggames.dozendays.TextButton;
	import com.blazinggames.dozendays.words.LetterTile;

	public class SlideGame
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const LEVEL_TIMES:Array = new Array(300,240,180);
		public static const LEVEL_POINTS_PER_LETTER:Array =new Array(5, 10, 20);
		public static const LEVEL_SECONDS_PER_LETTER:Array =new Array(8, 5, 3);
//		public static const SKIP_TEXT:String = "Skip";
		public static const DONE_TEXT:String = "EXIT!";
		public static const EMPTY_TEXT:String = "-";
		public static const WON_TEXT:String = "You Have WON!";
		public static const LOST_TEXT:String = "You Have Lost!";
		public static const SOUND_ON_TEXT:String = "Turn Sound off!";
		public static const SOUND_OFF_TEXT:String = "Turn Sound on!";

		public static const TOP_TEXT:String = "^";
		public static const LEFT_TEXT:String = "<";
		public static const RIGHT_TEXT:String = ">";
		public static const BOTTOM_TEXT:String = "v";
		
		public static const TILE_SIZE:int = 40;
		public static const NUM_ROWS:int = 10;
		public static const NUM_COLS:int = 10;

		public static const BACKGROUND_COLOR:uint = 0x000050;
		public static const TILE_COLOR:uint = 0x500050;
		public static const BORDER_COLOR:uint = 0x000030;
		public static const LETTER_COLOR:uint = 0xC000C0;
		public static const TILE_COLOR_DIM:uint = 0x200020;
		public static const LETTER_COLOR_DIM:uint = 0x500050;
		public static const HIGHLIGHT_COLOR:uint = 0xFF0050;
		public static const WORD_COLOR:uint = 0x00C000;
		public static const SLIDER_COLOR:uint = 0x010001;

		public static const PIXELS_PER_SECOND:int = 200;
		public static const REMOVAL_TIME_MS:int = 200;
		public static const ANIMATION_NONE:int = 0;
		public static const ANIMATION_SLIDING:int = 1;
		public static const ANIMATION_REMOVE:int = 2;
		public static const ANIMATION_DONE:int = 3;
		
// ***** VARIABLES **********************************************************

		[Embed(source="slide.mp3")]
		private var SlideSound:Class;
		public var _slideSound:Sound;
		[Embed(source="strum.mp3")]
		private var StrumSound:Class;
		public var _strumSound:Sound;
		[Embed(source="thud.mp3")]
		private var ThudSound:Class;
		public var _thudSound:Sound;
		protected var _soundState:Boolean

		protected var _slide:WordSlide;
		protected var _doneButton:TextButton;
		protected var _soundButton:TextButton;
		protected var _timerText:TextField;
		protected var _tiles:Array;

		protected var _backdrop:TextField;
		protected var _highScoreText:TextField;
		protected var _scoreText:TextField;
		
		protected var _pointsPerLetter:int;
		protected var _secondsPerLetter:int;
		protected var _timeToPlay:int;
		protected var _startTime:int;
		protected var _lastTime:int;
		protected var _score:int;
		protected var _highScore:int;
		protected var _slideList:SlideList;
//		protected var _selection:LetterTile;
		protected var _levelCompleteButton:TextButton;

		protected var _animationState:int;
		protected var _sliderX:int;
		protected var _sliderY:int;
		protected var _slidingTile:LetterTile;
		protected var _sliderPlaceholder:LetterTile;
		protected var _removeLettersHighlightEnd:int;

// ***** CONSTRUCTOR ********************************************************

		public function SlideGame(wshift:WordSlide)
		{
			var cntrRow:int, cntrCol:int, rowLength:int, tileX:int, tileY:int;
			_slide = wshift;
//			_selection = null;
			_slideSound = Sound(new SlideSound());
			_strumSound = Sound(new StrumSound());
			_thudSound = Sound(new ThudSound());
			_soundState = true;

			_backdrop = new TextField();
			_backdrop.multiline = true;
			_backdrop.width = 640;
			_backdrop.height = 480;
			_backdrop.textColor = 0xC000C0;
			_backdrop.background = true;
			_backdrop.backgroundColor = BACKGROUND_COLOR;
			addChild(_backdrop);
			
			// build board
			_tiles = new Array();
			var indx:int = 0;
			var wordFormat:TextFormat = new TextFormat(null, 20, WORD_COLOR, null, null, null, null, null, TextFormatAlign.CENTER);
			for (cntrRow = 0; cntrRow < NUM_ROWS; ++cntrRow) {
				tileX = 0 + TILE_SIZE;
				tileY = cntrRow * TILE_SIZE + TILE_SIZE;
				for (cntrCol = 0; cntrCol < NUM_COLS; ++cntrCol) {
					_tiles[indx] = new LetterTile(TILE_SIZE,TILE_SIZE, EMPTY_TEXT);
					_tiles[indx].x = tileX;
					_tiles[indx].y = tileY;
					_tiles[indx].setColors(LETTER_COLOR, TILE_COLOR);
					_tiles[indx].setBorder(true, BORDER_COLOR);
					_tiles[indx].addEventListener(LetterTile.TILE_SELECTED_EVENT, handleTileSelection);
					addChild(_tiles[indx]);

					tileX += TILE_SIZE;
					++indx;
				}
				
			}
			
			_sliderPlaceholder = new LetterTile(TILE_SIZE,TILE_SIZE, EMPTY_TEXT);
			_sliderPlaceholder.setColors(SLIDER_COLOR, SLIDER_COLOR);
			_sliderPlaceholder.setBorder(true, BORDER_COLOR);
			_sliderPlaceholder.visible = false;
			addChild(_sliderPlaceholder);

			_slidingTile = new LetterTile(TILE_SIZE,TILE_SIZE, EMPTY_TEXT);
			_slidingTile.setColors(LETTER_COLOR, TILE_COLOR);
			_slidingTile.setBorder(true, BORDER_COLOR);
			_slidingTile.visible = false;
			addChild(_slidingTile);

			_doneButton = new TextButton(DONE_TEXT, 160, 40);
			_doneButton.x = 480;
			_doneButton.y = 0;
			addChild(_doneButton);
			_doneButton.addEventListener(MouseEvent.MOUSE_UP, handleExit);

			_soundButton = new TextButton(SOUND_ON_TEXT, 160, 25);
			_soundButton.x = 480;
			_soundButton.y = 455;
			addChild(_soundButton);
			_soundButton.addEventListener(MouseEvent.MOUSE_UP, handleSoundToggle);

			_slideList = new SlideList();
			_slideList.x = 490;
			_slideList.y = 50;
			addChild(_slideList);

			var winFormat:TextFormat = new TextFormat(null, 50, 0xFFFF00, null, null, null, null, null, TextFormatAlign.CENTER);
			_highScoreText = new TextField();
			_highScoreText.width = 640;
			_highScoreText.height = 150;
			_highScoreText.y = 150;
			_highScoreText.defaultTextFormat = winFormat;
			_highScoreText.text = WON_TEXT;
			_highScoreText.visible = false;
			addChild(_highScoreText);

			_levelCompleteButton = new TextButton("Out of time! Click here to exit.", 640, 40);
			_levelCompleteButton.x = 0;
			_levelCompleteButton.y = 250;
			_levelCompleteButton.visible = false;
			addChild(_levelCompleteButton);
			_levelCompleteButton.addEventListener(MouseEvent.MOUSE_UP, handleExit);

			var scoreFormat:TextFormat = new TextFormat(null, 30, 0xF0F0FF, null, null, null, null, null, TextFormatAlign.CENTER);
			_scoreText = new TextField();
			_scoreText.width = 480;
			_scoreText.height = 40;
			_scoreText.x = 0;
			_scoreText.y = 440;
			_scoreText.defaultTextFormat = scoreFormat;
			_scoreText.text = "Score = 0";
			addChild(_scoreText);

			_timerText = new TextField();
			_timerText.width = 480;
			_timerText.height = 40;
			_timerText.x = 0;
			_timerText.y = 0;
			_timerText.defaultTextFormat = scoreFormat;
			addChild(_timerText);
			
			_highScore = 0;
		}

// ***** Functions - Puzzle Handling ****************************************

		public function startGame(playmode:int = 0):void
		{
			_animationState = ANIMATION_NONE;
			_score = 0;
			_scoreText.text = "Score = 0";
			_startTime = getTimer();
			_lastTime = _startTime;
			var lev:int = playmode % LEVEL_TIMES.length;
			startLevel(LEVEL_TIMES[lev], LEVEL_POINTS_PER_LETTER[lev], LEVEL_SECONDS_PER_LETTER[lev]);
		}
		
		private function startLevel(secs:int = 60, ppl:int = 5, spl:int = 1):void
		{
			var cntr:int;

			_pointsPerLetter = ppl;
			_secondsPerLetter = spl;
			_levelCompleteButton.visible = false;
			_highScoreText.visible = false;
			_timeToPlay = secs;

			_slideList.startLevel();
			// clear the game board
			for (cntr = 0; cntr < _tiles.length; ++cntr) {
				_tiles[cntr].setLetter(EMPTY_TEXT);
				_tiles[cntr].setColors(LETTER_COLOR, TILE_COLOR);
			}
			
			// add starting letters
			for (cntr = 0; cntr < _tiles.length; ++cntr)
				_tiles[cntr].setLetter(_slideList.getNextLetter());

			_sliderX = NUM_COLS - 1;
			_sliderY = NUM_ROWS - 1;
			_tiles[_sliderY*NUM_COLS+_sliderX].setColors(SLIDER_COLOR, SLIDER_COLOR);
		}

// ***** Functions - Event Handling *****************************************

		private function handleExit(evt:Event):void
		{
			_slide.changeMode(WordSlide.MODE_TITLE);
		}

		private function handleTileSelection(evt:Event):void
		{
			var cntr:int, indx:int = -1;
			for (cntr = 0; cntr < _tiles.length; ++cntr)
				if (evt.target == _tiles[cntr])
					indx = cntr;
			if (indx < 0)
				return;
			var selectedX:int = indx % NUM_COLS;
			var selectedY:int = indx / NUM_COLS;
			var deltaX:int = selectedX - _sliderX;
			var deltaY:int = selectedY - _sliderY;
			
			if (deltaX == deltaY) // clicking on slider should be ignored
				return
			if (deltaX == 0)
				if (deltaY < 0)
					slideUp();
				else
					slideDown();
			if (deltaY == 0)
				if (deltaX < 0)
					slideLeft();
				else
					slideRight();
		}

		private function handleSoundToggle(evt:Event):void
		{
			if (_soundState == true) {
				_soundState = false;
				_soundButton.setLabel(SOUND_OFF_TEXT)
				SoundMixer.soundTransform = new SoundTransform(0, 0);
			} else {
				_soundState = true;
				_soundButton.setLabel(SOUND_ON_TEXT)
				SoundMixer.soundTransform = new SoundTransform(1, 0);
			}
		}

// ***** Functions - game handling *******************************************

		private function moveSlider(targetX:int, targetY:int):void
		{
			if (_animationState != ANIMATION_NONE)
				return;
			var oldIndex:int = _sliderY * NUM_COLS + _sliderX;
			var newIndex:int = targetY * NUM_COLS + targetX;

			setLetter(_sliderX, _sliderY, getLetter(targetX, targetY));
			_tiles[oldIndex].setColors(LETTER_COLOR, TILE_COLOR);
			_tiles[newIndex].setColors(SLIDER_COLOR, SLIDER_COLOR);
			_sliderX = targetX;
			_sliderY = targetY;
			_slidingTile.setLetter(getLetter(targetX, targetY));
			_slidingTile.x = _tiles[newIndex].x;
			_slidingTile.y = _tiles[newIndex].y;
			_slidingTile.visible = true;

			_sliderPlaceholder.x = _tiles[oldIndex].x;
			_sliderPlaceholder.y = _tiles[oldIndex].y;
			_sliderPlaceholder.visible = true;
			_animationState = ANIMATION_SLIDING;
			_slideSound.play();
		}

		public function slideUp():void
		{
			if ((_sliderY - 1) < 0) {
				_thudSound.play()
				return;
			}
			moveSlider(_sliderX, _sliderY - 1);
		}

		public function slideDown():void
		{
			if ((_sliderY + 1) >= NUM_ROWS) {
				_thudSound.play()
				return;
			}
			moveSlider(_sliderX, _sliderY + 1);
		}

		public function slideLeft():void
		{
			if ((_sliderX - 1) < 0){
				_thudSound.play()
				return;
			}
			moveSlider(_sliderX - 1, _sliderY);
		}

		public function slideRight():void
		{
			if ((_sliderX + 1) >= NUM_COLS){
				_thudSound.play()
				return;
			}
			moveSlider(_sliderX + 1, _sliderY);
		}

		public function endTurn():void {
			var cntrRow:int, cntrCol:int, cntr:int;
			var wordLength:int;// = _slideList.getWordLength();
			var s:String;
			var scored:Boolean = false;
			var lettersRemoved:int = 0;
			var wordsRemoved:int = 0;
			
			// check for valid words and remove them
			for (cntrRow = 0; cntrRow < NUM_ROWS; ++cntrRow) {
				for (cntrCol = 0; cntrCol < NUM_COLS; ++cntrCol) {
					for (wordLength = SlideList.MIN_WORD_LENGTH; wordLength <= SlideList.MAX_WORD_LENGTH; ++wordLength) {
						// horizontal check
						s = getLetter(cntrCol, cntrRow);
						for (cntr = 1; cntr < wordLength; ++cntr)
							s += getLetter(cntrCol + cntr, cntrRow);
						if (_slideList.wasWordInList(s)) {
							for (cntr = 0; cntr < wordLength; ++cntr)
								flagLetter(cntrCol + cntr, cntrRow)
							scored = true;
							lettersRemoved += wordLength;
							++wordsRemoved;
						}
	
						// vertical check
						s = getLetter(cntrCol, cntrRow);
						for (cntr = 1; cntr < wordLength; ++cntr)
							s += getLetter(cntrCol, cntrRow + cntr);
						if (_slideList.wasWordInList(s)) {
							for (cntr = 0; cntr < wordLength; ++cntr)
								flagLetter(cntrCol, cntrRow + cntr);
							scored = true;
							lettersRemoved += wordLength;
							++wordsRemoved;
						}
					}
				}
			}
			if (scored) {
				_strumSound.play();
				var temp:int = wordsRemoved * _pointsPerLetter * lettersRemoved;
				_score += temp;
				_scoreText.text = "Score = " + _score;
				temp = lettersRemoved * _secondsPerLetter;
				_timeToPlay += temp;
				_removeLettersHighlightEnd = getTimer() + REMOVAL_TIME_MS;
				_animationState = ANIMATION_REMOVE;
				return;

			}
		}
		
		public function getLetter(tx:int, ty:int):String
		{
			if ( (tx < 0) || (tx >= NUM_COLS) || (ty < 0) || (ty >= NUM_ROWS) )
				return "*";
			var indx:int = ty * NUM_COLS + tx;
			return _tiles[indx].getLetter();
		}

		public function setLetter(tx:int, ty:int, s:String):void
		{
			if ( (tx < 0) || (tx >= NUM_COLS) || (ty < 0) || (ty >= NUM_ROWS) )
				return ;
			var indx:int = ty * NUM_COLS + tx;
			_tiles[indx].setLetter(s);
		}

		public function flagLetter(tx:int, ty:int, b:Boolean = true):void
		{
			if ( (tx < 0) || (tx >= NUM_COLS) || (ty < 0) || (ty >= NUM_ROWS) )
				return ;
			var indx:int = ty * NUM_COLS + tx;
			_tiles[indx].setFlagged(b);
			_tiles[indx].setColors(HIGHLIGHT_COLOR, LETTER_COLOR_DIM);
			
		}
		
		public function updateTime():void
		{
			if (_levelCompleteButton.visible)
				return;			
			var newTimer:int = getTimer();
			var playTime:int = _timeToPlay - (newTimer - _startTime) / 1000;
			var timeDelta:int = newTimer - _lastTime;
			var pixelAdjust:int = timeDelta * PIXELS_PER_SECOND / 1000;
			_lastTime = newTimer;

			switch (_animationState) {
			case ANIMATION_SLIDING:
				var deltaX:int = Math.abs(_sliderPlaceholder.x - _slidingTile.x);
				var multiplierX:int = (_sliderPlaceholder.x > _slidingTile.x) ? 1 : -1;
				var deltaY:int = Math.abs(_sliderPlaceholder.y - _slidingTile.y);
				var multiplierY:int = (_sliderPlaceholder.y > _slidingTile.y) ? 1 : -1;
				if ( (deltaX < pixelAdjust) && (deltaY < pixelAdjust)) {
					_animationState = ANIMATION_NONE;
					_sliderPlaceholder.visible = false;
					_slidingTile.visible = false;
					endTurn();
				} else {
					if (deltaX > 0)
						_slidingTile.x += (pixelAdjust * multiplierX);
					else
						_slidingTile.y += (pixelAdjust * multiplierY);

				}
				break;
			case ANIMATION_REMOVE:
				if (newTimer > _removeLettersHighlightEnd) {
					for (var cntr:int = 0; cntr < _tiles.length; ++cntr)
						if (_tiles[cntr].isFlagged()) {
							_tiles[cntr].setFlagged(false);
							_tiles[cntr].setColors(LETTER_COLOR, TILE_COLOR);
							_tiles[cntr].setLetter(_slideList.getNextLetter());
						}
					_animationState = ANIMATION_NONE;
				}
				break;
			default:
			}
			_timerText.text = "Time Remaining "+playTime;
			if ( (playTime < 0) && (_animationState == ANIMATION_NONE) ) {
				_timerText.text = "0";
				if (_score > _highScore) {
					_highScore = _score;
					_highScoreText.text = "You got the high score!";
				} else
					_highScoreText.text = "High Score is "+ _highScore;
				_levelCompleteButton.visible = true;
				_highScoreText.visible = true;
				_animationState = ANIMATION_DONE;
			}
		}
		
	}
}
